@@ -136,8 +136,7 @@ void xmlParserError(void *ctx, const char *msg, ...)
     /* Write in the actual message. */
   vsprintf(tmp, msg, args);
   va_end(args);
-  PROBLEM "XML Parsing Error: %s", buf
-  WARN;
+  Rf_warning("XML Parsing Error: %s", buf);
 #endif
 }
 
@@ -268,15 +267,13 @@ R_makeRefObject(void *ref, const char *className)
    SEXP klass, obj, sref;
 
    if(!ref) {
-      PROBLEM "NULL value for external reference"
-      WARN;
-      return(R_NilValue);
+       Rf_warning("NULL value for external reference");
+       return(R_NilValue);
    }
 
    PROTECT(klass = MAKE_CLASS((char *) className)); /* XXX define MAKE_CLASS with const */
    if(klass == R_NilValue) { /* Is this the right test? */
-      PROBLEM "Cannot find class %s for external reference", className
-      ERROR;
+       Rf_error("Cannot find class %s for external reference", className);
    }
    PROTECT(obj = NEW_OBJECT(klass));
    PROTECT(sref = R_MakeExternalPtr(ref, Rf_install(className), R_NilValue));
@@ -299,8 +296,7 @@ R_parseURI(SEXP r_uri)
   int i= 0;
   uri = xmlParseURI( CHAR( STRING_ELT( r_uri, 0 )));
   if(!uri) {
-     PROBLEM "cannot parse URI %s", CHAR( STRING_ELT( r_uri, 0) )
-     ERROR;
+      Rf_error("cannot parse URI %s", CHAR( STRING_ELT( r_uri, 0) ) );
   }
 
   PROTECT(ans = NEW_LIST(8));
@@ -340,7 +336,7 @@ RSXML_structuredStop(SEXP errorFun, xmlErrorPtr err)
     if(!err)
        n = 2;
 
-    PROTECT(e = allocVector(LANGSXP, 8));
+    PROTECT(e = allocVector(LANGSXP, n));
 
     SETCAR(e, errorFun != NULL && errorFun != R_NilValue ? errorFun :  Rf_install("xmlStructuredStop")); 
     ptr = CDR(e);
@@ -399,11 +395,11 @@ stop(const char *className, const char *msg, ...)
 
     PROTECT(e = allocVector(LANGSXP, 2));
     PROTECT(ns_name = mkString("XML"));
-    ns_env = R_FindNamespace(ns_name);
+    PROTECT(ns_env = R_FindNamespace(ns_name));
     SETCAR(e, findVarInFrame(ns_env, Rf_install("xmlStop")));
     SETCAR(CDR(e), error);
     Rf_eval(e, R_GlobalEnv);
-    UNPROTECT(2);
+    UNPROTECT(4);
 
 /*
     errorcall(error, "%s", msg);
